/* YOODA : Yet another Object Oriented Database
   Copyright (C) 1994  Eric Abecassis

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Library General Public
License as published by the Free Software Foundation; either
version 2 of the License, or (at your option) any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
Library General Public License for more details.

You should have received a copy of the GNU Library General Public
License along with this library; if not, write to the Free
Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA. */


/*****************************************************************************
******************************************************************************
**                                                                            
**        Y     Y   OOOO    OOOO   DDDDD     AA
**         Y   Y   O    O  O    O  D    D   A  A
**          Y Y    O    O  O    O  D    D  A    A 
**           Y     O    O  O    O  D    D  AAAAAA
**           Y     O    O  O    O  D    D  A    A
**           Y      OOOO    OOOO   DDDDD   A    A
**
******************************************************************************
******************************************************************************/
/*****************************************************************************
*
*  MODULE :             Iterator
*  SCOPE :              Iterator
*  FILE :               iter.cc
*  AUTHORS :            E. Abecassis
*  CREATION DATE :      August 1, 1993
*  COPYRIGHTS :        (C) E. Abecassis, 1993
*  HISTORY :
*
******************************************************************************/

#ifdef __GNUG__
#pragma implementation
#endif

/* IMPORTS *******************************************************************/

#ifndef __iter_HXX
#include "iter.hxx"
#endif

#ifndef __yooexcept_HXX
#include "yooexcept.hxx"
#endif

/*****************************************************************************/

/*= EXTERNAL FUNCTION =======================================================*/
GenIterator::GenIterator(Collection* c, int beg, int end)
/*=
[DESCRIPTION :]
  (IN)  
  (OUT) 
[DIAGNOSTIC :]
[CALL :]
=*/
: _scanCol(c) 
{
  this->reset(beg, end) ;
}


/*= EXTERNAL FUNCTION =======================================================*/
void GenIterator::reset(int beg, int end)
/*=
[DESCRIPTION :]
  (IN)  
  (OUT) 
[DIAGNOSTIC :]
[CALL :]
=*/
{
  int size = _scanCol == NULL ? 0 : _scanCol->count() ;
  if (size == 0) {
    _currPos = 0 ;
    _endPos = -1 ;
    return ;
  }
  if (end == -1) {
    end = _scanCol->count() - 1 ;
  }
  if ((beg >= size) || (beg < 0) || (end >= size) || (end < beg)) {
    call_throw(YOO_ERR_COL, "indice out of bounds") ;
  }
        
  _beginPos = beg ;
  _endPos = end + 1 ;
  _currPos = beg ;  
  if (_scanCol != NULL) {
    this->_currLeaf = _scanCol->scanLeaf(_beginPos,_localPos,_localEnd) ;
  }   
}


/*= EXTERNAL FUNCTION =======================================================*/
void GenIterator::next()
/*=
[DESCRIPTION :]
  (IN)  
  (OUT) 
[DIAGNOSTIC :]
[CALL :]
=*/
{
  if (++_currPos < _endPos) {
    if (_localPos++ >= _localEnd) {
      _currLeaf = _scanCol->scanLeaf(_currPos, _localPos, _localEnd) ;
    }
  }
}


/*= EXTERNAL FUNCTION =======================================================*/
void GenIterator::prev()
/*=
[DESCRIPTION :]
  (IN)  
  (OUT) 
[DIAGNOSTIC :]
[CALL :]
=*/
{
  if (_currPos-- > _beginPos) {
    if (_localPos-- <= 0) {
      _currLeaf = _scanCol->scanLeaf(_currPos, _localPos, _localEnd) ;
    }
  }
}


/*= EXTERNAL FUNCTION =======================================================*/
void GenIterator::go(int pos)
/*=
[DESCRIPTION :]
  (IN)  
  (OUT) 
[DIAGNOSTIC :]
[CALL :]
=*/
{
  if ( (pos < _beginPos) || (pos >= _endPos) ) return ;
  _currPos = pos ;
  this->_currLeaf = _scanCol->scanLeaf(_currPos, _localPos, _localEnd) ;
}


